/* strip control chars & VT330 "ANSI" escape sequences
** w.j.m. 1990, after VT330 PRM Vol.1, using GNU FLEX 2.3-6
*/

/*      Remove control characters (except BEL, BS, HT, VT, FF) and
**      DEC VT3** control sequences.
**      Convert non-redundant CR into newline,
**      and force a newline to precede VT and FF.
**
**      usage:
**              stripcc [-v] [-b] [<inputfile] [>outputfile]
**
**      options:
**              -v      add newline for explicit VT3** cursor positioning
**                      commands (not for BS!)
**
**              -b[CCC] add a newline when a line exceeds "CCC" bytes,
**                      default CCC = 132
*/

%{

#define DEFAULT_MAXLINE 132

#define USAGE   fprintf(stderr,\
"Usage: STRIPCC [-v] [-b[lll]] [<input] [>output]")


static int /*logical*/ vopt = 0, bopt = 0;
static int maxline;                     /* used with -b */

static int /*logical*/ strip_nl = 0;    /* set if \n has just been added */
static int linelen = 0;                 /* # bytes in current line */

main(argc,argv)
int argc; char **argv;
{
        int iarg;

#ifdef VAXC
        redirect(&argc,&argv);
#endif
        for(iarg = 1; iarg < argc; iarg++) {
                if(argv[iarg][0] == '-') {
                        switch(argv[iarg][1]) {
                          case 'v':
                                vopt = 1;
                                break;
                          case 'b':
                                bopt = 1;
                                if(sscanf(argv[iarg]+2,"%d",&maxline) == 0 ||
                                   maxline <= 0)
                                        maxline = DEFAULT_MAXLINE;
                                break;
                          default:
                                USAGE; exit(4);
                        }
                } else {
                        USAGE; exit(4);
                }
        }

        while (yylex()) ;
}

#define output(s) fputs(s,yyout)

/*****/

#define force_newline do {\
        if(linelen > 0) {output("\n"); linelen=0; strip_nl++;}\
        } while(0)

#define bopt_check_line do {\
        if(bopt && linelen >= maxline) {output("\n"); linelen=0; strip_nl++;}\
        } while(0)
%}

%x ST_PENDING

        /* C0 controls (except for printable and BEL) */
C0_CC   [\000-\006\016-\032\034-\037\177]
ESC     \033
SUB     \032

        /* C1 controls & 7-bit equivalents */
C1_CC   [\200-\217\221-\232]|({ESC}[\@-OQ-Z])
DCS     \220|({ESC}P)
CSI     \233|({ESC}\[)
ST      \234|({ESC}\\)
OSC     \235|({ESC}\])
PM      \236|({ESC}\^)
APC     \237|({ESC}_)

        /* "escape sequences" incl. badly formed ones */
ESC_SEQ {ESC}[\040-\057]*[\060-\176]?

        /* "control sequences" for VT*** cursor positioning */
CTL_SEQ_v       {CSI}[\060-\077]*[\040-\057]*[ABCDHf]
        /* remaining "control sequences" incl. badly formed ones */
        /* NOTE: "A" is \101, originally last char was [\100-\176] */
CTL_SEQ         {CSI}[\060-\077]*[\040-\057]*[\100E-GI-eg-\176]?

%%

{C0_CC}         |
{C1_CC}         |
{ST}            |
{ESC_SEQ}       |
{CTL_SEQ}       ;

{CTL_SEQ_v}     if(vopt) force_newline;

{DCS}           |
{OSC}           |
{PM}            |
{APC}           BEGIN(ST_PENDING);

        /* don't echo anything while waiting for ST (or VT330 equivalent) */
        /* if DCS, OSC, PM, APC come here, we just remain in this state */
<ST_PENDING>{ST}        |
<ST_PENDING>{SUB}       |
<ST_PENDING>{C1_CC}     |
<ST_PENDING>{CTL_SEQ}   BEGIN(0);

<ST_PENDING>{CTL_SEQ_v} BEGIN(0); if(vopt) force_newline;

<ST_PENDING>\n          |
<ST_PENDING>.           ;

\n              if(!strip_nl) ECHO; strip_nl = 0; linelen = 0;

\v              |
\f              force_newline; ECHO; linelen += yyleng; bopt_check_line;

\r              force_newline;                  /* don't echo */

.               ECHO; linelen += yyleng; strip_nl = 0; bopt_check_line;
